@@ -16,16 +16,22 @@
 
 #include "driver.h"
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
+
 PhonedevConfig::PhonedevConfig() :
 Keydata("/bayonne/phone")
 {
-	static KEYDEF defkeys[] = {
+	static Keydata::Define defkeys[] = {
 	{"devices", "16"},
 	{"wink", "100"},
 	{"buffers", "1"},
+	{"echo", "low"},
 	{NULL, NULL}};
 
-	Load(defkeys);
+	load(defkeys);
 }
 
 size_t PhonedevConfig::getStack(void)
@@ -51,17 +57,17 @@ Driver()
 
 	thread_count = thread_select = 0;
 	
-	slog(SLOG_INFO) << "Generic Phone Device (phonedev) driver loaded; capacity=" << port_count << endl;
+	slog(Slog::levelInfo) << "Generic Phone Device (phonedev) driver loaded; capacity=" << port_count << endl;
 }
 
 PhonedevDriver::~PhonedevDriver()
 {
-	Stop();
+	stop();
 	delete ports;
 	delete groups;
 }
 
-int PhonedevDriver::Start(void)
+int PhonedevDriver::start(void)
 {
 	char path[32];
 	int fd, ts;
@@ -70,11 +76,11 @@ int PhonedevDriver::Start(void)
 
 	if(active)
 	{
-		slog(SLOG_ERROR) << "driver already started" << endl;
+		slog(Slog::levelError) << "driver already started" << endl;
 		return 0;
 	}
 
-	slog(SLOG_INFO) << "driver starting " << scount << " service thread(s)" << endl;
+	slog(Slog::levelInfo) << "driver starting " << scount << " service thread(s)" << endl;
 	threads = new PhonedevService[scount];
 
 	for(ts = 0; ts < port_count; ++ts)
@@ -93,7 +99,7 @@ int PhonedevDriver::Start(void)
 	return count;
 }
 
-void PhonedevDriver::Stop(void)
+void PhonedevDriver::stop(void)
 {
 	int id;
 	if(!active)
@@ -111,7 +117,7 @@ void PhonedevDriver::Stop(void)
 	delete[] threads;
 	
 	active = false;
-	slog(SLOG_INFO) << "driver stopping service thread(s)" << endl;
+	slog(Slog::levelInfo) << "driver stopping service thread(s)" << endl;
 }
 
 Trunk *PhonedevDriver::getTrunkPort(int id)
@@ -130,7 +136,7 @@ PhonedevService *PhonedevDriver::getService(void)
 	return svc;
 }
 
-void PhonedevDriver::Notify(unsigned char id)
+void PhonedevDriver::notify(unsigned char id)
 {
 	TrunkEvent event;
 	PhonedevTrunk *trunk;
@@ -147,3 +153,7 @@ void PhonedevDriver::Notify(unsigned char id)
 }
 
 PhonedevDriver phivr;
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
