@@ -32,6 +32,18 @@
 #endif
 #endif
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+#endif
+
+typedef	enum
+{
+	ECHO_OFF = 0,
+	ECHO_LOW,
+	ECHO_MEDIUM,
+	ECHO_HIGH
+} aeclevel_t;
+
 typedef	enum
 {
 	CPA_DIALTONE = 0,
@@ -79,6 +91,8 @@ private:
 	PhonedevTrunk(int fd, int ts);
 	~PhonedevTrunk();
 
+	void initSyms(void);
+	void setEcho(aeclevel_t level);
 	void setCPADetect(cpatone_t tone, bool flag);
 	void getName(char *buffer);
 	unsigned long getIdleTime(void);
@@ -90,8 +104,10 @@ private:
 	bool seizeHandler(TrunkEvent *event);
 	bool ringHandler(TrunkEvent *event);
 	bool waitHandler(TrunkEvent *event);
+	bool loadHandler(TrunkEvent *event);
 	bool dialHandler(TrunkEvent *event);
 	bool playHandler(TrunkEvent *event);
+	bool playwaitHandler(TrunkEvent *event);
 	bool recordHandler(TrunkEvent *event);
 	bool sleepHandler(TrunkEvent *event);
 	bool hangupHandler(TrunkEvent *event);
@@ -105,8 +121,8 @@ private:
 	void getEvents(void);
 	void setTimer(timeout_t timeout = 0);
 	void incTimer(timeout_t timeout);
-	void Exit(void);
-	void TrunkStep(trunkstep_t step);
+	void exit(void);
+	void trunkStep(trunkstep_t step);
 
 public:
 	inline int getDevice(void)
@@ -127,19 +143,19 @@ private:
 public:
 	PhonedevRecord(PhonedevTrunk *trunk);
 	~PhonedevRecord();
-	void Initial(void);
-	void Run(void);
+	void initial(void);
+	void run(void);
 };
 
-class PhonedevPlay : private AudioFile, public Service
+class PhonedevPlay : private URLAudio, public Service
 {
 private:
 	int dev;
 	bool reset;
 	driver_t driver;
 	size_t bufsize, samples;
-	void Initial(void);
-	void Run(void);
+	void initial(void);
+	void run(void);
 	char *getContinuation(void);
 
 public:
@@ -159,10 +175,10 @@ private:
 	int count;
 	PhonedevTrunk *first, *last;
 
-	void Attach(PhonedevTrunk *trunk);
-	void Detach(PhonedevTrunk *trunk);
-	void Run(void);
-	void Update(unsigned char id = 0xff);
+	void attach(PhonedevTrunk *trunk);
+	void detach(PhonedevTrunk *trunk);
+	void run(void);
+	void update(unsigned char id = 0xff);
 
 	PhonedevService();
 	~PhonedevService();
@@ -195,8 +211,8 @@ private:
 	PhonedevTrunk **ports;
 	int port_count;
 
-	int Start(void);
-	void Stop(void);
+	int start(void);
+	void stop(void);
 
 public:
 	PhonedevDriver();
@@ -208,7 +224,11 @@ public:
 	Trunk *getTrunkPort(int id);
 	PhonedevService *getService(void);
 	aaScript *getScript(void);
-	void Notify(unsigned char id);    // ALWAYS trunk + 1
+	void notify(unsigned char id);    // ALWAYS trunk + 1
 };
 
 extern PhonedevDriver phivr;
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
