@@ -17,6 +17,17 @@
 #include "driver.h"
 #include <sys/ioctl.h>
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
+
+void PhonedevTrunk::initSyms(void)
+{
+	setConst(SYM_INTERFACE, "analog");
+	setConst(SYM_NETWORK, "none");
+}
+
 bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 {
 	char script[256];
@@ -34,8 +45,11 @@ bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 	switch(event->id)
 	{
 	case TRUNK_ENTER_STATE:
-		debug->DebugState(this, "idle");
-		ScriptSymbol::Purge();
+		synctimer = exittimer = 0;
+		enterState("idle");
+		setIdle(true);
+		ScriptSymbol::purge();
+		setEcho(ECHO_OFF);
 		setCPADetect(CPA_DIALTONE, false);
 		setCPADetect(CPA_BUSYTONE, false);
 		flags.dsp = DSP_MODE_INACTIVE;
@@ -52,20 +66,20 @@ bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 
 		flags.trunk = TRUNK_MODE_INACTIVE;
 		status[id] = '-';
-		time(&idle);
-		Purge();
+		time(&idletime);
+		purge();
 		rings = 0;
 		switch(driver)
 		{
 #ifdef	IXJ_PHONEDEV
 		case ixj_driver:
-			close(dev);
+			::close(dev);
 			strcpy(script, "/dev/");
 			getName(script + 5);
 			dev = open(script, O_RDWR);
 			if(dev < 0)
 			{
-				slog(SLOG_CRITICAL) << script + 5 << ": device reset failed" << endl;
+				slog(Slog::levelCritical) << script + 5 << ": device reset failed" << endl;
 				raise(SIGINT);
 			}
 #ifdef	POSIX_PHONEDEV
@@ -98,9 +112,10 @@ bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 		req = group->getRequest();
 		if(req)
 		{
+			setConst(SYM_PARENT, req->getParent());
 			argv = req->getList();
 			start = *(argv++);
-			if(Attach(start))
+			if(attach(start))
 			{
 				flags.trunk = TRUNK_MODE_OUTGOING;
 				setList(argv);
@@ -111,6 +126,7 @@ bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 				delete req;
 				return true;
 			}
+			ScriptSymbol::purge();
 			delete req;
 		}
 		chk = atoi(group->chkRequest());
@@ -133,16 +149,9 @@ bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 		rings = 0;
 		if(!start)
 			start = group->getSchedule(script);
-		if(Attach(start))
+		if(attach(start))
 		{
-			while(event->parm.argv[argc])
-			{
-				strcpy(script, event->parm.argv[argc++]);
-				data = strtok_r(script, "=", &sp);
-				value = strtok_r(NULL, "=", &sp);
-				if(data && value)
-					setConstant(data, value);
-			}
+			setList(&event->parm.argv[argc]);
 			flags.ready = false;
 			endTimer();		
 			if(flags.trunk == TRUNK_MODE_OUTGOING)
@@ -157,10 +166,30 @@ bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 			}
 			return true;
 		}
+		ScriptSymbol::purge();
 		flags.trunk = TRUNK_MODE_INACTIVE;
-		slog(SLOG_ERROR) << "phone" << id << ": " << start << ": cannot start" << endl;
-		return true;		
+		slog(Slog::levelError) << "phone" << id << ": " << start << ": cannot start" << endl;
+		return true;
+	case TRUNK_RING_REDIRECT:
+		start = group->getRedirect(event->parm.argv[0], script);
+		rings = 0;
+		if(attach(start))
+		{
+			setConst(SYM_REDIRECT, event->parm.argv[0]);
+			setList(&event->parm.argv[1]);
+			flags.ready = false;
+			flags.trunk = TRUNK_MODE_INCOMING;
+			endTimer();
+			handler = &PhonedevTrunk::stepHandler;
+			group->incIncoming();		
+			return true;
+		}
+		ScriptInterp::purge();
+		slog(Slog::levelError) << "phone" << id << ": " << event->parm.argv[0] << ": unable to redirect" << endl;
+		return true;
 	case TRUNK_RINGING_ON:
+		if(!group->getAnswer())
+			return true;
 		rings = 1;
 		flags.trunk = TRUNK_MODE_INCOMING;
 		group->incIncoming();
@@ -171,3 +200,7 @@ bool PhonedevTrunk::idleHandler(TrunkEvent *event)
 	}
 	return false;
 }
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
