@@ -16,7 +16,12 @@
 
 #include "driver.h"
 
-void PhonedevTrunk::TrunkStep(trunkstep_t step)
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
+
+void PhonedevTrunk::trunkStep(trunkstep_t step)
 {
 	TrunkEvent event;
 	
@@ -33,6 +38,8 @@ bool PhonedevTrunk::stepHandler(TrunkEvent *event)
 	switch(event->id)
 	{
 	case TRUNK_MAKE_STEP:
+		if(idleHangup())
+			return true;
 		switch(event->parm.step)
 		{
 		case TRUNK_STEP_SLEEP:
@@ -59,6 +66,14 @@ bool PhonedevTrunk::stepHandler(TrunkEvent *event)
 		case TRUNK_STEP_DIALXFER:
 			handler = &PhonedevTrunk::dialHandler;
 			return true;
+		case TRUNK_STEP_PLAYWAIT:
+			handler = &PhonedevTrunk::playwaitHandler;
+			return true;
+#ifdef	XML_SCRIPTS
+		case TRUNK_STEP_LOADER:
+			handler = &PhonedevTrunk::loadHandler;
+			return true;
+#endif
 		}
 		return false;
 	case TRUNK_SERVICE_SUCCESS:
@@ -81,6 +96,7 @@ bool PhonedevTrunk::stepHandler(TrunkEvent *event)
 		}
 		else
 			setCPADetect(CPA_DIALTONE, false);
+		setEcho(ECHO_OFF);
 		setCPADetect(CPA_BUSYTONE, false);
 		setDTMFDetect();
 		endTimer();
@@ -89,18 +105,20 @@ bool PhonedevTrunk::stepHandler(TrunkEvent *event)
 		// for the dsp to settle before stepping
 		if(flags.reset || thread)
 		{
-			debug->DebugState(this, "reset");
+			enterState("reset");
 			if(thread)
-				setTimer(thread->Stop());
+				setTimer(thread->stop());
 			else
 				setTimer(keythreads.getResetDelay());
 			return true;
 		}
-		debug->DebugStep(this, getScript());
+		strcpy(numbers[5].sym.data, "step");
+		debug->debugStep(this, getScript());
+		monitor->monitorStep(this, getScript());
 
 		// step and delay if still in same handler
 
-		ScriptStep();
+		scriptStep();
 
 		if(handler == &PhonedevTrunk::stepHandler)
 			setTimer(keythreads.getStepDelay());
@@ -109,3 +127,7 @@ bool PhonedevTrunk::stepHandler(TrunkEvent *event)
 	}
 	return false;
 }
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
