@@ -17,6 +17,11 @@
 #include <sys/ioctl.h>
 #include "driver.h"
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
+
 char PhonedevTrunk::status[240];
 
 PhonedevTrunk::PhonedevTrunk(int fd, int ts) :
@@ -91,7 +96,7 @@ Trunk(ts), TimerPort()
 		cpatone[CPA_DIALTONE].enable = 0;
 		ioctl(dev, IXJCTL_SET_FILTER, &cpatone[0]);
 		
-		slog(SLOG_INFO) << "phone" << id << ": Quicknet driver started" << endl;
+		slog(Slog::levelInfo) << "phone" << id << ": Quicknet driver started" << endl;
 #ifdef	POSIX_PHONEDEV
 		ioctl(dev, IXJCTL_PSTN_LINETEST, &ctrl);
 #else
@@ -99,16 +104,16 @@ Trunk(ts), TimerPort()
 #endif
 		if(!ctrl)
 		{
-			slog(SLOG_WARNING) << "phone" << id << ": linetest failure" << endl;
+			slog(Slog::levelWarning) << "phone" << id << ": linetest failure" << endl;
 			handler = &PhonedevTrunk::busyHandler;
 		}
 		break;
 #endif
 	default:
-		slog(SLOG_INFO) << "phone" << id << ": generic driver started" << endl;
+		slog(Slog::levelInfo) << "phone" << id << ": generic driver started" << endl;
 	}
 	service = phivr.getService();		
-	service->Attach(this);
+	service->attach(this);
 	
 	(this->*handler)(&event);
 }
@@ -117,7 +122,7 @@ PhonedevTrunk::~PhonedevTrunk()
 {
 	handler = NULL;
 	if(service)
-		service->Detach(this);
+		service->detach(this);
 
 	endTimer();
 	
@@ -136,7 +141,32 @@ PhonedevTrunk::~PhonedevTrunk()
 #endif
 	}
 	close(dev);
-	slog(SLOG_INFO) << "phone" << id << ": device stopped" << endl;
+	slog(Slog::levelInfo) << "phone" << id << ": device stopped" << endl;
+}
+
+void PhonedevTrunk::setEcho(aeclevel_t level)
+{
+	switch(driver)
+	{
+#ifdef	IXJ_PHONEDEV
+	case ixj_driver:
+		switch(level)
+		{
+		case ECHO_OFF:
+			ioctl(dev, IXJCTL_AEC_STOP);
+			break;
+		case ECHO_LOW:
+			ioctl(dev, IXJCTL_AEC_START, AEC_LOW);
+			break;
+		case ECHO_MEDIUM:
+			ioctl(dev, IXJCTL_AEC_START, AEC_MED);
+			break;
+		case ECHO_HIGH:
+			ioctl(dev, IXJCTL_AEC_START, AEC_HIGH);
+		}
+		break;
+#endif
+	}
 }
 
 void PhonedevTrunk::setCPADetect(cpatone_t tone, bool mode)
@@ -159,8 +189,15 @@ void PhonedevTrunk::getName(char *buffer)
 	sprintf(buffer, "phone%d", id);
 }
 
-void PhonedevTrunk::Exit(void)
+void PhonedevTrunk::exit(void)
 {
+	if(!flags.onexit)
+		if(redirect("::exit"))
+		{
+			flags.onexit = true;
+			return;
+		}
+
 	handler = &PhonedevTrunk::hangupHandler;
 }
 
@@ -169,44 +206,58 @@ bool PhonedevTrunk::postEvent(TrunkEvent *event)
 	bool rtn = true;
 	trunkhandler_t prior;
 
-	EnterMutex();
+	enterMutex();
 	switch(event->id)
 	{
+	case TRUNK_TIMER_EXIT:
+		if(!exittimer)
+			rtn = false;
+		exittimer = 0;
+		break;
+	case TRUNK_TIMER_SYNC:
+		if(!synctimer)
+			rtn = false;
+		synctimer = 0;
+		break;
 	case TRUNK_TIMER_EXPIRED:
 		if(!getTimer())
 			rtn = false;
 		break;
 	case TRUNK_DTMF_KEYUP:
+		if(flags.offhook)
+			time(&idletime);
 		if(!flags.dtmf)
 			rtn = false;
 		break;
 	}	
 	if(!rtn)
 	{
-		LeaveMutex();
+		leaveMutex();
 		return false;
 	}
 
 	if(!handler)
 	{
-		slog(SLOG_WARNING) << "phone" << id;
+		slog(Slog::levelWarning) << "phone" << id;
 		slog() << ": no handler active; event=" << event->id << endl;
-		LeaveMutex();
+		leaveMutex();
 		return false;
 	}
 
 retry:
-	debug->DebugEvent(this, event);
+	debug->debugEvent(this, event);
 	prior = handler;
 	rtn = (this->*handler)(event);
 	if(rtn)
 	{
 		if(handler != prior)
 		{
+			if(prior == &PhonedevTrunk::idleHandler)
+				setIdle(false);
 			event->id = TRUNK_ENTER_STATE;
 			goto retry;
 		}
-		LeaveMutex();
+		leaveMutex();
 		return true;
 	}
 
@@ -230,20 +281,52 @@ retry:
 			break;
 	case TRUNK_CPA_DIALTONE:
 	case TRUNK_STOP_DISCONNECT:
-		TrunkSignal(TRUNK_SIGNAL_HANGUP);
+		if(flags.onexit)
+			break;
+		trunkSignal(TRUNK_SIGNAL_HANGUP);
 		event->id = TRUNK_STOP_STATE;
 		goto retry;
 		break;
+	case TRUNK_SEND_MESSAGE:
+		if(recvEvent(event))
+		{
+			event->id = TRUNK_STOP_STATE;
+			goto retry;
+		}
+		break;
 	case TRUNK_TIMER_EXPIRED:
-		TrunkSignal(TRUNK_SIGNAL_TIMEOUT);
+		trunkSignal(TRUNK_SIGNAL_TIMEOUT);
 		event->id = TRUNK_STOP_STATE;
 		goto retry;
 		break;
+        case TRUNK_TIMER_SYNC:
+                if(trunkSignal(TRUNK_SIGNAL_TIME))
+                {
+                        event->id = TRUNK_STOP_STATE;
+                        goto retry;
+                }
+                break;
+        case TRUNK_TIMER_EXIT:
+                if(trunkSignal(TRUNK_SIGNAL_TIME))
+                        event->id = TRUNK_STOP_STATE;
+                else
+                        event->id = TRUNK_STOP_DISCONNECT;
+                goto retry;
+
+	case TRUNK_CHILD_EXIT:
+		if(!isActive())
+			break;
+		if(trunkSignal(TRUNK_SIGNAL_CHILD))
+		{
+			event->id = TRUNK_STOP_STATE;
+			goto retry;
+		}
+		break;
 	case TRUNK_DTMF_KEYUP:
-		if(digits < 32)
+		if(digits < MAX_DIGITS)
 			dtmf.bin.data[digits++] = digit[event->parm.dtmf.digit];
 		dtmf.bin.data[digits] = 0;
-		if(TrunkSignal((trunksignal_t)(event->parm.dtmf.digit + TRUNK_SIGNAL_0)))
+		if(trunkSignal((trunksignal_t)(event->parm.dtmf.digit + TRUNK_SIGNAL_0)))
 		{
 			event->id = TRUNK_STOP_STATE;
 			goto retry;
@@ -262,7 +345,7 @@ retry:
 		handler = &PhonedevTrunk::busyHandler;
 		break;
 	case TRUNK_MAKE_IDLE:
-		handler = &PhonedevTrunk::idleHandler;
+		handler = &PhonedevTrunk::hangupHandler;
 		break;
 	default:
 		rtn = false;
@@ -272,7 +355,7 @@ retry:
 		event->id = TRUNK_ENTER_STATE;
 		goto retry;
 	}
-	LeaveMutex();
+	leaveMutex();
 	return rtn;
 }
 
@@ -313,7 +396,7 @@ void PhonedevTrunk::getEvents(void)
 	if(exc.bits.pstn_ring)
 	{
 		event.id = TRUNK_RINGING_ON;
-		cp = getSymbol("ringid");
+		cp = getSymbol(SYM_RINGID);
 		if(cp)
 			event.parm.ring.digit = atoi(cp);
 		else
@@ -340,20 +423,20 @@ void PhonedevTrunk::getEvents(void)
 		switch(ctrl)
 		{
 		case PSTN_ON_HOOK:
-//			slog(SLOG_DEBUG) << "onhook" << endl;
+//			slog(Slog::levelDebug) << "onhook" << endl;
 			event.id = TRUNK_ON_HOOK;
 			postEvent(&event);
 			break;
 		case PSTN_RINGING:
-//			slog(SLOG_DEBUG) << "ringing" << endl;
+//			slog(Slog::levelDebug) << "ringing" << endl;
 			break;
 		case PSTN_OFF_HOOK:
-//			slog(SLOG_DEBUG) << "offhook" << endl;
+//			slog(Slog::levelDebug) << "offhook" << endl;
 			event,id = TRUNK_OFF_HOOK;
 			postEvent(&event);
 			break;
 //		case PSTN_PULSE_DIAL:
-//			slog(SLOG_DEBUG) << "pulse dial" << endl;
+//			slog(Slog::levelDebug) << "pulse dial" << endl;
 		}
 	}
 	if(exc.bits.dtmf_ready)
@@ -461,13 +544,13 @@ void PhonedevTrunk::setTimer(timeout_t ptimer)
 {
 	TimerPort::setTimer(ptimer);
 	if(!service->isThread())
-		service->Update();
+		service->update();
 }
 
 void PhonedevTrunk::incTimer(timeout_t ptimer)
 {
 	TimerPort::incTimer(ptimer);
-	service->Update();
+	service->update();
 }
 
 unsigned long PhonedevTrunk::getIdleTime(void)
@@ -476,7 +559,7 @@ unsigned long PhonedevTrunk::getIdleTime(void)
 
 	time(&now);
 	if(handler == &PhonedevTrunk::idleHandler)
-		return now - idle;
+		return now - idletime;
 
 	return 0;
 }
@@ -518,4 +601,6 @@ struct phone_capability *PhonedevTrunk::getCapability(phone_cap cid, int sub)
 	return NULL;
 }
 
-
+#ifdef	CCXX_NAMESPACES
+};
+#endif
