@@ -16,18 +16,25 @@
 
 #include "driver.h"
 
+#ifdef	CCXX_NAMESPACES
+using namespace std;
+namespace ost {
+#endif
+
 bool VPBTrunk::collectHandler(TrunkEvent *event)
 {
 	timeout_t wakeup;
 	unsigned short mask;
+	unsigned copy = 0;
+	Symbol *sym = (Symbol *)data.collect.var;
 
 	switch(event->id)
 	{
 	case TRUNK_ENTER_STATE:
-		debug->DebugState(this, "collect");
+		enterState("collect");
 		if(digits >= data.collect.count)
 		{
-			TrunkSignal(TRUNK_SIGNAL_STEP);
+			trunkSignal(TRUNK_SIGNAL_STEP);
 			handler = &VPBTrunk::stepHandler;
 			return true;
 		}
@@ -36,18 +43,14 @@ bool VPBTrunk::collectHandler(TrunkEvent *event)
 		setTimer(data.collect.timeout);
 		return true;
 	case TRUNK_TIMER_EXPIRED:
-		if(!TrunkSignal(TRUNK_SIGNAL_TIMEOUT))
-			TrunkSignal(TRUNK_SIGNAL_STEP);
+		if(!trunkSignal(TRUNK_SIGNAL_TIMEOUT))
+			trunkSignal(TRUNK_SIGNAL_STEP);
 		handler = &VPBTrunk::stepHandler;
 		return true;
 	case TRUNK_DTMF_KEYUP:
-		endTimer();
 		mask = (1 << event->parm.dtmf.digit);
 		if(mask & data.collect.ignore)
-		{
-			setTimer(data.collect.timeout);
 			return true;
-		}
 		if(!(mask & data.collect.term))
 		{
 			dtmf.bin.data[digits++] = digit[event->parm.dtmf.digit];
@@ -55,10 +58,31 @@ bool VPBTrunk::collectHandler(TrunkEvent *event)
 		}				
 		if(digits >= data.collect.count || (mask & data.collect.term))
 		{
-			TrunkSignal(TRUNK_SIGNAL_STEP);
+                        if(sym)
+                        {
+                                copy = sym->flags.size;
+                                if(copy > digits)
+                                        copy = digits;
+                                if(copy)
+                                        strncpy(sym->data, dtmf.bin.data, copy);
+                                sym->data[copy] = 0;
+                                if(sym->flags.commit)
+                                        commit(sym);
+				digits = 0;
+				dtmf.bin.data[0] = 0;
+                        }
+
+			endTimer();
+			trunkSignal(TRUNK_SIGNAL_STEP);
 			handler = &VPBTrunk::stepHandler;
 		}
+		else
+			setTimer(data.collect.timeout);
 		return true;
 	}
 	return false;
 }
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
