@@ -16,6 +16,11 @@
 
 #include "driver.h"
 
+#ifdef	CCXX_NAMESPACES
+using namespace std;
+namespace ost {
+#endif
+
 bool VPBTrunk::hangupHandler(TrunkEvent *event)
 {
 	timeout_t reset;
@@ -23,20 +28,22 @@ bool VPBTrunk::hangupHandler(TrunkEvent *event)
 	switch(event->id)
 	{
 	case TRUNK_ENTER_STATE:
-		debug->DebugState(this, "hangup");
+		synctimer = exittimer = 0;
+		enterState("hangup");
 		status[id] = 'h';
 		endTimer();
+		disjoin();
 		if(tgi.pid)
 			kill(tgi.pid, SIGHUP);
 		if(thread)
-			reset = thread->Stop();
+			reset = thread->stop();
 		else
 			reset = 0;
 		setDTMFDetect(false);
 		flags.dsp = DSP_MODE_INACTIVE;
 		flags.offhook = false;
 		vpb_sethook_async(handle, VPB_ONHOOK);
-		Detach();
+		detach();
 		if(group->getHangup() > reset)
 			setTimer(group->getHangup());
 		else
@@ -48,6 +55,9 @@ bool VPBTrunk::hangupHandler(TrunkEvent *event)
 		handler = &VPBTrunk::idleHandler;
 		return true;
 	case TRUNK_MAKE_IDLE:
+	case TRUNK_STOP_DISCONNECT:
+	case TRUNK_LINE_WINK:
+	case TRUNK_CPA_DIALTONE:
 		return true;
 	case TRUNK_RINGING_ON:
 		endTimer();
@@ -57,3 +67,7 @@ bool VPBTrunk::hangupHandler(TrunkEvent *event)
 	}
 	return false;
 }
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
