@@ -16,7 +16,12 @@
 
 #include "driver.h"
 
-void VPBTrunk::TrunkStep(trunkstep_t step)
+#ifdef	CCXX_NAMESPACES
+using namespace std;
+namespace ost {
+#endif
+
+void VPBTrunk::trunkStep(trunkstep_t step)
 {
 	TrunkEvent event;
 	
@@ -33,11 +38,16 @@ bool VPBTrunk::stepHandler(TrunkEvent *event)
 	switch(event->id)
 	{
 	case TRUNK_MAKE_STEP:
+		if(idleHangup())
+			return true;
 		switch(event->parm.step)
 		{
 		case TRUNK_STEP_SLEEP:
 			handler = &VPBTrunk::sleepHandler;
 			return true;
+		case TRUNK_STEP_TONE:
+			handler = &VPBTrunk::toneHandler;
+			return true;
 		case TRUNK_STEP_HANGUP:
 			handler = &VPBTrunk::hangupHandler;
 			return true;
@@ -50,14 +60,26 @@ bool VPBTrunk::stepHandler(TrunkEvent *event)
 		case TRUNK_STEP_FLASH:
 			handler = &VPBTrunk::flashonHandler;
 			return true;
+		case TRUNK_STEP_PLAYWAIT:
+			handler = &VPBTrunk::playwaitHandler;
+			return true;
 		case TRUNK_STEP_PLAY:
 			handler = &VPBTrunk::playHandler;
 			return true;
 		case TRUNK_STEP_RECORD:
 			handler = &VPBTrunk::recordHandler;
 			return true;
+#ifdef	XML_SCRIPTS
+		case TRUNK_STEP_LOADER:
+			handler = &VPBTrunk::loadHandler;
+			return true;
+#endif
 		case TRUNK_STEP_DIALXFER:
-//			handler = &VPBTrunk::dialHandler;
+		  //  ===== not to comment out
+	      		handler = &VPBTrunk::dialHandler;
+			return true;
+		case TRUNK_STEP_JOIN:
+			handler = &VPBTrunk::joinHandler;
 			return true;
 		}
 		return false;
@@ -85,18 +107,20 @@ bool VPBTrunk::stepHandler(TrunkEvent *event)
 		// for the dsp to settle before stepping
 		if(flags.reset || thread)
 		{
-			debug->DebugState(this, "reset");
+			enterState("reset");
 			if(thread)
-				setTimer(thread->Stop());
+				setTimer(thread->stop());
 			else
 				setTimer(keythreads.getResetDelay());
 			return true;
 		}
-		debug->DebugStep(this, getScript());
+		strcpy(numbers[5].sym.data, "step");
+		debug->debugStep(this, getScript());
+		monitor->monitorStep(this, getScript());
 
 		// step and delay if still in same handler
 
-		ScriptStep();
+		scriptStep();
 
 		if(handler == &VPBTrunk::stepHandler)
 			setTimer(keythreads.getStepDelay());
@@ -105,3 +129,7 @@ bool VPBTrunk::stepHandler(TrunkEvent *event)
 	}
 	return false;
 }
+
+#ifdef	CCXX_NAMESPACES
+};
+#endif
