@@ -17,33 +17,60 @@
 #include "bayonne.h"
 #include <iomanip>
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
+
+static RETSIGTYPE sigfault(int signo)
+{
+        Debug::stackTrace(signo);
+        exit(signo);
+}
+
 class TraceDebug : public Debug
 {
 public:
 	TraceDebug();
-	void DebugState(Trunk *trunk, char *state);
-	void DebugScript(Trunk *trunk, char *msg);
-	void DebugStep(Trunk *trunk, scriptline_t *line);
+	bool debugTest(void);
+	bool debugFinal(int signo);
+	void debugState(Trunk *trunk, char *state);
+	void debugScript(Trunk *trunk, char *msg);
+	void debugStep(Trunk *trunk, Line *line);
 } trace;
 
 TraceDebug::TraceDebug() :
 Debug()
 {
-	slog(SLOG_INFO) << "debug: trace module loaded" << endl;
+	slog(Slog::levelInfo) << "debug: trace module loaded" << endl;
 }
 
-void TraceDebug::DebugScript(Trunk *trunk, char *msg)
+bool TraceDebug::debugTest(void)
+{
+	signal(SIGSEGV, sigfault);
+        signal(SIGABRT, sigfault);
+
+	return false;
+}
+
+bool TraceDebug::debugFinal(int signo)
+{
+	Debug::stackTrace(signo);
+        return false;
+}
+
+void TraceDebug::debugScript(Trunk *trunk, char *msg)
 {
 	char buffer[32];
 	char *value;
 
-	EnterMutex();
+	enterMutex();
 	trunk->getName(buffer);
-	slog(SLOG_DEBUG) << buffer << ": " << msg << endl;
-	LeaveMutex();
+	slog(Slog::levelDebug) << buffer << ": " << msg << endl;
+	leaveMutex();
 }
 
-void TraceDebug::DebugStep(Trunk *trunk, scriptline_t *line)
+void TraceDebug::debugStep(Trunk *trunk, Line *line)
 {
 	int i;
 
@@ -52,12 +79,12 @@ void TraceDebug::DebugStep(Trunk *trunk, scriptline_t *line)
 
 	if(!line)
 	{
-		slog(SLOG_DEBUG) << buffer << ": exit" << endl;
+		slog(Slog::levelDebug) << buffer << ": exit" << endl;
 		return;
 	}
 
-	EnterMutex();
-	slog(SLOG_DEBUG) << buffer << ": step ";
+	enterMutex();
+	slog(Slog::levelDebug) << buffer << ": step ";
 	slog() << setbase(16) << line->mask << " " << setbase(10);
 	slog() << line->cmd << "(";
 	for(i = 0; i < line->argc; ++i)
@@ -67,16 +94,19 @@ void TraceDebug::DebugStep(Trunk *trunk, scriptline_t *line)
 		slog() << line->args[i];
 	}
 	slog() << ")" << endl;	
-	LeaveMutex();
+	leaveMutex();
 }
 
-void TraceDebug::DebugState(Trunk *trunk, char *state)
+void TraceDebug::debugState(Trunk *trunk, char *state)
 {
 	char buffer[32];
 	trunk->getName(buffer);
 
-	EnterMutex();
-	slog(SLOG_DEBUG) << buffer << ": " << state << endl;
-	LeaveMutex();
+	enterMutex();
+	slog(Slog::levelDebug) << buffer << ": " << state << endl;
+	leaveMutex();
 }
 
+#ifdef	CCXX_NAMESPACES
+};
+#endif
