@@ -15,22 +15,43 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include <bayonne.h>
-#include <ctype.h>
+#include <cc++/strchar.h>
+#ifdef	HAVE_SSTREAM
+#include <sstream>
+#else
+#include <strstream>
+#endif
+
+#ifdef	HAVE_SSTREAM
+#define	strstream	ostringstream
+#endif
+
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
 
 class EnglishTranslator : public Translator
 {
 private:
 	char *getName(void)
 		{return "english";};
+protected:
+	bool _have_am_pm;
 public:
 	EnglishTranslator();
-	char *Speak(Trunk *trunk);
+	char *speak(Trunk *trunk);
 } english;
 
 EnglishTranslator::EnglishTranslator() :
-Translator("/bayonne/english", keylocal.getLast("english"))
+Translator("/bayonne/english")
 {
-	slog(SLOG_DEBUG) << "TTS: loading english phrasebooks..." << endl;
+	const char *ampm;
+
+	slog(Slog::levelDebug) << "TTS: loading english phrasebooks..." << endl;
+	ampm = getLast("haveampm");
+	_have_am_pm = ampm != 0 && ampm[0] != 'F' && ampm[0] != 'f' &&
+		ampm[0] != 'N' && ampm[0] != 'n';
 }
 
 static void spell(strstream *s, const char *text);
@@ -54,9 +75,9 @@ static void order(strstream *s, int num)
 	if(num > 19)
 	{
 		if(num % 10)
-			*s << ',' << num / 10;
+			*s << ',' << num / 10 * 10;
 		else
-			*s << ',' << num / 10 << "th";
+			*s << ',' << num << "th";
 		num %= 10;
 	}
 	if(num)
@@ -85,7 +106,7 @@ static void lownumber(strstream *s, int num)
 static bool number(strstream *s, const char *text)
 {
 	unsigned long num;
-	bool zero = false;
+	bool zero = true;
 
 	if(!text)
 		return true;
@@ -101,21 +122,21 @@ static bool number(strstream *s, const char *text)
 	num = atol(text);
 	if(num > 999999999)
 	{
-		zero = true;
+		zero = false;
 		lownumber(s, num / 1000000000);
 		*s << ",billion";
 		num %= 1000000000;
 	}
 	if(num > 999999)
 	{
-		zero = true;
+		zero = false;
 		lownumber(s, num / 1000000);
 		*s << ",million";
 		num %= 1000000;
 	}
 	if(num > 999)
 	{
-		zero = true;
+		zero = false;
 		lownumber(s, num / 1000);
 		*s << ",thousand";
 		num %= 1000;
@@ -140,7 +161,6 @@ static void spell(strstream *s, const char *text)
 	if(!text)
 		return;
 
-
 	while((ch = *(text++)))
 	{
 		if(ch >= 'A' && ch <= 'Z')
@@ -155,6 +175,36 @@ static void spell(strstream *s, const char *text)
 	}
 }
 
+static void sayweekday(strstream *s, const char *text)
+{
+	static char *days[] = {
+		"sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"};
+
+        time_t cl;
+        struct tm *ti;
+	int month, day, year;
+
+        time(&cl);
+        ti = localtime (&cl);
+
+	if(text[2] == '/')
+		sscanf(text, "%02d/%02d/%04d", &month, &day, &year);
+	else if(text[2] == '.')
+		sscanf(text, "%02d.%02d.%04d", &day, &month, &year);
+	else
+		sscanf(text, "%04d%02d%02d", &year, &month, &day);
+
+	ti->tm_year = year;
+	ti->tm_mon = month - 1;
+	ti->tm_mday = day;
+
+        if(ti->tm_year > 1000) 
+		ti->tm_year -= 1900;
+
+        mktime(ti);
+	*s << ',' << days[ti->tm_wday];
+}
+	
 static void saydate(strstream *s, const char *text)
 {
 	static char *months[] = {
@@ -366,7 +416,7 @@ static bool sayduration(strstream *s, const char *text)
 	return true;
 }
 			
-static void saytime(strstream *s, const char *text)
+static void saytime(strstream *s, const char *text, bool have_am_pm)
 {
 	int hour = 0, minute = 0;
 	bool pm = false;
@@ -389,25 +439,41 @@ static void saytime(strstream *s, const char *text)
 	number(s, buf);
 	if(minute)
 	{
+		if(minute < 10)
+			*s << ",o";
 		sprintf(buf, "%d", minute);
 		number(s, buf);
 	}
 	if(pm)
-		*s << ",p,m";
+		*s << ",p";
 	else
-		*s << ",a,m";
+		*s << ",a";
+	if (have_am_pm) {
+		*s << "_m";
+	} else {
+		*s << ",m";
+	}
 }
 
-char *EnglishTranslator::Speak(Trunk *trunk)
+char *EnglishTranslator::speak(Trunk *trunk)
 {
 	char *pbuf = getPlayBuffer(trunk);
+#ifdef	HAVE_SSTREAM
+	ostringstream str;
+#else
 	strstream str(pbuf, 254);
+#endif
 	char rules[256];
 	const char *rule = NULL;
 	const char *phrase;
 	bool skip = false;
 	long lastnum = 0;
 	char *text;
+#ifdef	SCRIPT_LOCAL_DEFINE
+	ScriptImage *cmd = trunk->getImage();
+#else
+	ScriptCommand *cmd = trunk->getCommand();
+#endif
 
 	while(NULL != (rule = trunk->getValue(NULL)))
 	{
@@ -417,6 +483,18 @@ char *EnglishTranslator::Speak(Trunk *trunk)
 			continue;
 		}
 		phrase = getLast(rule + 1);
+#ifdef	SCRIPT_PREPROCESSOR_OVERRIDE
+		if(!phrase)
+		{
+			snprintf(rules, 256, "_english_%s", rule + 1);
+			phrase = cmd->getLast(rules);
+		}
+		if(!phrase)
+		{
+			snprintf(rules, 256, "_all_%s", rule + 1);
+			phrase = cmd->getLast(rules);
+		}
+#endif
 		if(!phrase)
 			phrase = rule;
 
@@ -430,7 +508,7 @@ char *EnglishTranslator::Speak(Trunk *trunk)
 				if(skip)
 					return NULL;
 			}
-			else if(!stricmp(rule, "&ignore"))
+			else if(!stricmp(rule, "&ignore") || !stricmp(rule, "&several"))
 			{
 				trunk->getValue(NULL);
 			}
@@ -490,7 +568,7 @@ char *EnglishTranslator::Speak(Trunk *trunk)
 				skip = false;
 				text = trunk->getValue(NULL);
 				if(text)
-					saytime(&str, text);
+					saytime(&str, text, _have_am_pm);
 			}
 			else if(!stricmp(rule, "&date"))
 			{
@@ -499,6 +577,13 @@ char *EnglishTranslator::Speak(Trunk *trunk)
 				if(text)
 					saydate(&str, text);
 			}
+			else if(!stricmp(rule, "&weekday"))
+			{
+				skip = false;
+				text = trunk->getValue(NULL);
+				if(text)
+					sayweekday(&str, text);
+			}
 			else if(!stricmp(rule, "&unit"))
 			{
 				skip = false;
@@ -556,24 +641,33 @@ char *EnglishTranslator::Speak(Trunk *trunk)
 			}
 			else 
 			{
+#if 0
 				skip = false;
 				str << ',' << rule;
+#else
+				return "unsupported speak tag";
+#endif
 			}
 			rule = strtok(NULL, " ;,\t\n");
-		}	
+		}
 	}
 
+#ifdef	HAVE_SSTREAM
+	snprintf(pbuf, 256, "%s", str.str().c_str());
+#else
 	str << ends;
 	pbuf[255] = 0;
+#endif
+
 	while(*pbuf)
 	{
 		*pbuf = tolower(*pbuf);
 		++pbuf;
 	}
+
 	return NULL;
 }
 	
-
-	
-	
-
+#ifdef	CCXX_NAMESPACES
+};
+#endif
