@@ -15,7 +15,21 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include <bayonne.h>
-#include <ctype.h>
+#include <cc++/strchar.h>
+#ifdef	HAVE_SSTREAM
+#include <sstream>
+#else
+#include <strstream>
+#endif
+
+#ifdef	HAVE_SSTREAM
+#define	strstream	ostringstream
+#endif
+
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
 
 class FrenchTranslator : public Translator
 {
@@ -24,13 +38,13 @@ private:
 		{return "french";};
 public:
 	FrenchTranslator();
-	char *Speak(Trunk *trunk);
+	char *speak(Trunk *trunk);
 } french;
 
 FrenchTranslator::FrenchTranslator() :
-Translator("/bayonne/french", keylocal.getLast("french"))
+Translator("/bayonne/french")
 {
-	slog(SLOG_DEBUG) << "TTS: loading french phrasebooks..." << endl;
+	slog(Slog::levelDebug) << "TTS: loading french phrasebooks..." << endl;
 }
 
 static void spell(strstream *s, const char *text);
@@ -199,6 +213,37 @@ static void spell(strstream *s, const char *text)
 	}
 }
 
+static void sayweekday(strstream *s, const char *text)
+{
+	static char *days[] = {
+		"sunday", "monday", "tuesday", "wednesday", "thursday",
+		"friday", "saturday"};
+
+        time_t cl;
+        struct tm *ti;
+	int month, day, year;
+
+        time(&cl);
+        ti = localtime (&cl);
+
+	if(text[2] == '/')
+		sscanf(text, "%02d/%02d/%04d", &month, &day, &year);
+	else if(text[2] == '.')
+		sscanf(text, "%02d.%02d.%04d", &day, &month, &year);
+	else
+		sscanf(text, "%04d%02d%02d", &year, &month, &day);
+
+	ti->tm_year = year;
+	ti->tm_mon = month - 1;
+	ti->tm_mday = day;
+
+        if(ti->tm_year > 1000) 
+		ti->tm_year -= 1900;
+
+        mktime(ti);
+	*s << ',' << days[ti->tm_wday];
+}
+
 static void saydate(strstream *s, const char *text)
 {
 	static char *months[] = {
@@ -412,16 +457,25 @@ static void saytime(strstream *s, const char *text)
 		*s << ",0";
 }
 
-char *FrenchTranslator::Speak(Trunk *trunk)
+char *FrenchTranslator::speak(Trunk *trunk)
 {
 	char *pbuf = getPlayBuffer(trunk);
+#ifdef	HAVE_SSTREAM
+	ostringstream str;
+#else
 	strstream str(pbuf, 254);
+#endif
 	char rules[256];
 	const char *rule = NULL;
 	const char *phrase;
 	bool skip = false;
 	long lastnum = 0;
 	char *text;
+#ifdef	SCRIPT_LOCAL_DEFINE
+	ScriptImage *cmd = trunk->getImage();
+#else
+	ScriptCommand *cmd = trunk->getCommand();
+#endif
 
 	while(NULL != (rule = trunk->getValue(NULL)))
 	{
@@ -431,6 +485,18 @@ char *FrenchTranslator::Speak(Trunk *trunk)
 			continue;
 		}
 		phrase = getLast(rule + 1);
+#ifdef	SCRIPT_PREPROCESSOR_OVERRIDE
+		if(!phrase)
+		{
+			snprintf(rules, 256, "_french_%s", rule + 1);
+			phrase = cmd->getLast(rules);
+		}
+		if(!phrase)
+		{
+			snprintf(rules, 256, "_all_%s", rule + 1);
+			phrase = cmd->getLast(rules);
+		}
+#endif
 		if(!phrase)
 			phrase = rule;
 
@@ -444,7 +510,7 @@ char *FrenchTranslator::Speak(Trunk *trunk)
 				if(skip)
 					return NULL;
 			}
-			else if(!stricmp(rule, "&ignore"))
+			else if(!stricmp(rule, "&ignore") || !stricmp(rule, "&several"))
 			{
 				trunk->getValue(NULL);
 			}
@@ -513,6 +579,13 @@ char *FrenchTranslator::Speak(Trunk *trunk)
 				if(text)
 					saydate(&str, text);
 			}
+			else if(!stricmp(rule, "&weekday"))
+			{
+				skip = false;
+				text = trunk->getValue(NULL);
+				if(text)
+					sayweekday(&str, text);
+			}
 			else if(!stricmp(rule, "&unit"))
 			{
 				skip = false;
@@ -577,8 +650,12 @@ char *FrenchTranslator::Speak(Trunk *trunk)
 		}	
 	}
 
+#ifdef	HAVE_SSTREAM
+	snprintf(pbuf, 256, "%s", str.str().c_str());
+#else
 	str << ends;
 	pbuf[255] = 0;
+#endif
 	while(*pbuf)
 	{
 		*pbuf = tolower(*pbuf);
@@ -587,7 +664,9 @@ char *FrenchTranslator::Speak(Trunk *trunk)
 	return NULL;
 }
 	
-
+#ifdef	CCXX_NAMESPACES
+};
+#endif
 	
 	
 
