@@ -1,4 +1,4 @@
-// Copyright (C) 2000 Open Source Telecom Corporation.
+// Copyright (C) 2000-2001 Open Source Telecom Corporation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,72 +16,203 @@
 
 #include "server.h"
 #include <pwd.h>
+#include <grp.h>
 #include <math.h>
+#include <dlfcn.h>
+
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
 
 #define	LIB_VERSION	"/" VERPATH "/"
 
 KeyLocal::KeyLocal() :
 Keydata("/bayonne/localize")
 {
-	static KEYDEF keydefs[] = {
+	static Keydata::Define keydefs[] = {
 	{"primarycurrency", "dollar"},
 	{"primarychange", "cent"},
 	{"convertcurrency", "1.00"},
 	{NULL, NULL}};
 
-	Load(keydefs);
+	load(keydefs);
+}
+
+#ifdef	XML_SCRIPTS
+
+KeyProxy::KeyProxy() :
+Keydata("/bayonne/proxy")
+{
+	static Keydata::Define keydefs[] = {
+	{"timeout", "0"},
+	{NULL, NULL}};
+
+	load("~bayonne/proxy");
+	load(keydefs);
+}
+
+const char *KeyProxy::getHTTPServer(void)
+{
+	const char *cp = getLast("httpserver");
+	if(!cp)
+		cp = getLast("server");
+	return cp;
+}
+
+tpport_t KeyProxy::getHTTPPort(void)
+{
+	const char *cp;
+
+	if(!getHTTPServer())
+		return 0;
+
+	cp = getLast("httpport");
+	if(!cp)
+		cp = getLast("port");
+	if(cp)
+		return atoi(cp);
+
+	return 80;
+}
+
+timeout_t KeyProxy::getTimeout(void)
+{
+	return 1000l * atoi(getLast("timeout"));
+}	
+
+#endif
+
+KeyImports::KeyImports() :
+Keydata("/bayonne/imports")
+{
+	static Keydata::Define defimports[] = {
+	{"perl", ""},
+	{"python", ""},
+	{NULL, NULL}};
+
+	load("~bayonne/imports");
+	load(defimports);
 }
 
 KeyPaths::KeyPaths() :
 Keydata("/bayonne/paths")
 {
-	static KEYDEF defpaths[] = {
-	{"libpath", "/usr/lib/bayonne"},
-	{"libexec", "/usr/lib"},
-	{"tgipath", "/usr/lib/tgi:/bin:/usr/bin"},
-	{"datafiles", "/var/bayonne"},
-	{"scripts", "/usr/share/aascripts"},
-	{"prompts", "/usr/share/aaprompts"},
+	static Keydata::Define defpaths[] = {
+//	{"libpath", "/usr/lib/bayonne"},
+//	{"libexec", "/usr/libexec"},
+//	{"tgipath", "/usr/libexec/bayonne:/bin:/usr/bin"},
+//	{"datafiles", "/var/bayonne"},
+//	{"scripts", "/usr/share/aascripts"},
+//	{"prompts", "/usr/share/aaprompts"},
 	{"spool", "/var/spool/bayonne"},
-	{"runfiles", "/var/run"},
+	{"runfiles", "/var/run/bayonne"},
 	{"precache", "/var/bayonne/cache"},
-	{"config", "bayonne runtime configuration"},
+	{"logpath", "/var/log/bayonne"},
+	{"sox", "/usr/bin/sox"},
 	{NULL, NULL}};
 
-	Load("~bayonne/paths");
-	Load(defpaths);
+	load("~bayonne/paths");
+	load(defpaths);
 }
 
-KeyFeed::KeyFeed() :
-Keydata("/bayonne/audiofeed")
+void KeyPaths::setPrefix(char *prefix)
 {
-	static KEYDEF defkeys[] = {
-	{"buffers", "4"},
-	{"format", "ulaw"},
-#ifdef	__FreeBSD__
-	{"stack", "2"},
-#endif
+	char tmpfs[65];
+	char *pp = prefix + strlen(prefix);
+	const char *cp;
+
+	cp = getLast("libpath");
+	if(!cp)
+	{
+		strcpy(pp, "/../lib/bayonne");
+		setValue("libpath", prefix);
+	}
+
+	cp = getLast("libexec");
+	if(!cp)
+	{
+		strcpy(pp, "/../libexec");
+		setValue("libexec", prefix);
+	}
+	cp = getLast("tgipath");
+	if(!cp)
+	{
+		strcpy(pp, "/../libexec/bayonne:/bin:/usr/bin");
+		setValue("tgipath", prefix);
+	}
+
+	cp = getLast("scripts");
+	if(!cp)
+	{
+		strcpy(pp, "/../share/aascripts");
+		setValue("scripts", prefix);
+	}
+
+	cp = getLast("prompts");
+	if(!cp)
+	{
+		strcpy(pp, "/../share/aaprompts");
+		setValue("prompts", prefix);
+	}
+
+	cp = getLast("tmpfs");
+	if(!cp)
+		cp = "/dev/shm";
+
+	if(!isDir(cp))
+		cp = "/tmp";
+
+	snprintf(tmpfs, sizeof(tmpfs), "%s/.bayonne", cp);
+	setValue("tmpfs", tmpfs);
+	setValue("tmp", "/tmp/.bayonne");
+
+	cp = getenv("CONFIG_KEYDATA");
+	if(!cp)
+		cp = ETC_PREFIX;
+	setValue("etc", cp);
+
+	*pp = 0;
+}
+	
+KeyHandlers::KeyHandlers() :
+Keydata("/bayonne/handlers")
+{
+	static Keydata::Define defpaths[] = {
+	{"tts", "festival"},
+	{"url", "wget"},
+	{"timeout", "90"},
 	{NULL, NULL}};
 
-	Load("~bayonne/audiofeeds");
-	Load(defkeys);
+	load("~bayonne/handlers");
+	load(defpaths);
 }
 
 KeyServer::KeyServer() :
 Keydata("/bayonne/server")
 {
-	static KEYDEF defkeys[] = {
-	{"user", "mail"},
+	static Keydata::Define defkeys[] = {
+	{"user", "bayonne"},
+	{"group", "bayonne"},
 	{"default", "default"},
-	{"nodes", "12"},
+	{"nodes", "1"},
 	{"token", "&"},
+	{"password", "fts"},
 	{NULL, NULL}};
 
 	struct passwd *pwd;
+	struct group *grp;
 	char namebuf[128];
+	char prefix[256];
 	char *cp;
 
-	Load("~bayonne/server");
+	strcpy(prefix, getenv("HOME"));
+	strcat(prefix, "/.bayonne");
+	altdir = strdup(prefix);
+	phrdir = "~.bayonne/phrases/";
+
+	load("~bayonne/server");
+
 	if(!getLast("node"))
 	{
 		gethostname(namebuf, sizeof(namebuf) - 1);
@@ -90,60 +221,110 @@ Keydata("/bayonne/server")
 			*cp = 0;
 		setValue("node", namebuf);
 	}
-	Load(defkeys);
+	load(defkeys);
 
 	uid = getuid();
 	gid = getgid();
-	if(uid)
-		return;
 
 	pwd = getpwnam(getLast("user"));
 	if(pwd)
+		gid = pwd->pw_gid;
+
+	if(pwd && !uid)
 	{
 		uid = pwd->pw_uid;
-		gid = pwd->pw_gid;
+		strcpy(prefix, pwd->pw_dir);
+		strcat(prefix, "/apps");
+		altdir = strdup(prefix);
+		phrdir = "~phrases/";
 	}
-	else
+	if(!pwd && !uid)
 	{
 		uid = 0xffff;
 		gid = 0xffff;
 	}
+
+#ifdef	USER_HOSTING
+	if(getuid())
+		grp = NULL;
+	else
+	{
+		grp = getgrnam(getLast("group"));
+		if(!grp)
+			endgrent();
+	}
+	if(grp)
+	{
+		while(*grp->gr_mem)
+		{
+			cp = *(grp->gr_mem++);
+			pwd = getpwnam(cp);
+			if(pwd)
+			{
+				snprintf(prefix, sizeof(prefix), "%s/.bayonne", pwd->pw_dir);
+				if(isDir(prefix))
+					keyusers.setValue(cp, prefix);
+			}
+		}
+	}
 	endpwent();
+	if(grp)
+		endgrent();
+#endif
 }
 
-void KeyServer::loadGroups(void)
+void KeyServer::loadGroups(bool test)
 {
-	char *group;
+	char *group, *tok;
+	char buffer[256];
 
 	new TrunkGroup();
-	slog(SLOG_DEBUG) << "loading default trunk group" << endl;
+	slog(Slog::levelDebug) << "loading default trunk group" << endl;
 	
 	group = (char *)TrunkGroup::first->getLast("groups");
-	if(!group)
+	if(!group || test)
 		return;
 
-	group = strtok(group, " \t\n");
+	strcpy(buffer, group);
+	group = strtok_r(buffer, " \t\n", &tok);
 	while(group)
 	{
-		slog(SLOG_DEBUG) << "loading " << group << " trunk group" << endl;
+		slog(Slog::levelDebug) << "loading " << group << " trunk group" << endl;
 		new TrunkGroup(group);
-		group = strtok(NULL, " \t\n");
+		group = strtok_r(NULL, " \t\n", &tok);
 	}
 }
 
+void KeyServer::setUid(void)
+{
+	setuid(uid);
+	uid = getuid();
+}
+
+void KeyServer::setGid(void)
+{
+	setgid(gid);
+	gid = getgid();
+}
+
 KeyThreads::KeyThreads() :
 Keydata("/bayonne/threads")
 {
-	static KEYDEF defpaths[] = {
+	static Keydata::Define defpaths[] = {
+	{"audit", "0"},
 	{"audio", "0"},
-	{"feed", "0"},
 	{"priority", "0"},
 	{"gateways", "0,1"},
 	{"services", "1,1"},
+	{"database", "0"},
 	{"network", "0"},
+#ifdef	XML_SCRIPTS
+	{"xml", "0"},
+#endif
 	{"managers", "0"},
 	{"scheduler", "0"},
 	{"gui", "0"},
+	{"rtp", "0"},
 	{"resetdelay", "18"},
 	{"stepdelay", "36"},
 	{"stepinterval", "18"},
@@ -161,7 +342,7 @@ Keydata("/bayonne/threads")
 	if(cp)
 		setValue("priority", cp);
 
-	Load(defpaths);
+	load(defpaths);
 }
 
 int KeyThreads::getPolicy()
@@ -178,19 +359,25 @@ int KeyThreads::getPolicy()
 	return 0;
 }
 
+#ifdef	NODE_SERVICES
+
 KeyNetwork::KeyNetwork() :
-Keydata("/bayonne/database")
+Keydata("/bayonne/network")
 {
-	static KEYDEF defkeys[] = {
-	{"databasedelay", "8"},
+	static Keydata::Define defkeys[] = {
 	{"refresh", "5"},
+	{"live", "25"},
+	{"elect", "120"},
+	{"expire", "300"},
+	{"database", "127.0.0.1:7002"},
 	{"address", "127.0.0.1:7001"},
 	{"broadcast", "127.255.255.255"},
-	{"database0", "127.0.0.1:7000"},
+	{"monitor", "127.0.0.1:7070"},
 	{NULL, NULL}};
 
-	Load("~bayonne/database");
-	Load(defkeys);
+	load("~bayonne/network");
+
+	load(defkeys);
 }
 
 InetAddress KeyNetwork::getAddress(void)
@@ -205,6 +392,30 @@ InetAddress KeyNetwork::getAddress(void)
 	return InetAddress(buffer);
 }
 
+InetAddress KeyNetwork::getMonitorAddress(void)
+{
+	char buffer[65];
+	char *cp;
+
+	strcpy(buffer, getLast("monitor"));
+	cp = strchr(buffer, ':');
+	if(*cp)
+		*cp = 0;
+	return InetAddress(buffer);
+}
+
+InetHostAddress KeyNetwork::getDBHost(void)
+{
+	char buffer[65];
+	char *cp;
+
+	strcpy(buffer, getLast("database"));
+	cp = strchr(buffer, ':');
+	if(*cp)
+		*cp = 0;
+	return InetHostAddress(buffer);
+}
+
 InetHostAddress KeyNetwork::getBroadcast(void)
 {
 	return InetHostAddress(getLast("broadcast"));
@@ -222,68 +433,41 @@ tpport_t KeyNetwork::getPort(void)
 	return 0;
 }
 
-tpport_t KeyNetwork::getDatabasePort(unsigned id)
+tpport_t KeyNetwork::getMonitorPort(void)
 {
 	char buffer[65];
-	char adapter[12];
 	char *cp;
 
-	sprintf(adapter, "database%u", id);
-	cp = (char *)getLast(adapter);
-	if(!cp)
-		return 0;
-
-	strcpy(buffer, cp);
-	cp = strchr(cp, ':');
-	if(!cp)
-		return 0;
-
-	return atoi(++cp);
-}
-
-InetHostAddress KeyNetwork::getDatabase(unsigned id)
-{
-	char buffer[65];
-	char adapter[12];
-	char *cp;
-
-	sprintf(adapter, "database%u", id);
-	cp = (char *)getLast(adapter);
-	if(!cp)
-		return InetHostAddress("0.0.0.0");
-
-	strcpy(buffer, cp);
+	strcpy(buffer, getLast("monitor"));
 	cp = strchr(buffer, ':');
-	if(*cp)
-		*cp = 0;
-	return InetHostAddress(buffer);
+	if(cp)
+		return atoi(++cp);
+	return 0;
 }
-	
-KeyMailbox::KeyMailbox() :
-Keydata("/bayonne/mailbox")
-{
-	static KEYDEF defkeys[] = {
-	{"count", "99"},
-	{"limit", "30"},
-	{"quota", "30"},
-	{"minimum", "150"},
-	{"maximum", "300"},
-	{"password", "999"},
-	{NULL, NULL}};
 
-	Load("~bayonne/mailbox");
-	Load(defkeys);
+tpport_t KeyNetwork::getDBPort(void)
+{
+	char buffer[65];
+	char *cp;
+
+	strcpy(buffer, getLast("database"));
+	cp = strchr(buffer, ':');
+	if(cp)
+		return atoi(++cp);
+	return 0;
 }
 
+#endif
+
 KeyMemory::KeyMemory() :
 Keydata("/bayonne/memory")
 {
-	static KEYDEF defpaths[] = {
+	static Keydata::Define defpaths[] = {
 	{"symbols", "64"},
 	{"page", "1024"},
 	{NULL, NULL}};
 
-	Load(defpaths);
+	load(defpaths);
 }
 
 size_t KeyThreads::getStack(void)
@@ -365,8 +549,6 @@ int KeyThreads::getGateways(void)
 Plugins::Plugins() :
 Keydata("/bayonne/plugins")
 {
-	Load("~bayonne/plugins");
-
 	if(getLast("driver"))
 		return;
 
@@ -377,12 +559,14 @@ Keydata("/bayonne/plugins")
 #else
 	setValue("driver", "dummy");
 #endif
-	
+	pidcount = 0;	
 }
 
 Plugins::~Plugins()
 {
-	dynunload();
+//	dynunload();
+	while(pidcount)
+		kill(pids[--pidcount], SIGTERM);
 }
 
 char *Plugins::getDriverName(void)
@@ -410,6 +594,7 @@ void Plugins::loadManagers(void)
 {
 	char path[256];
 	char list[412];
+	char *argv[3];
 	char *cp;
 
 	cp = (char *)getLast("managers");
@@ -427,15 +612,25 @@ void Plugins::loadManagers(void)
 			path[0] = 0;
 		else
 		{
-			strcpy(path, keypaths.getLibpath());
-			strcat(path, LIB_VERSION);
+			strcpy(path, keypaths.getLibexec());
+			strcat(path, "/");
 		}
+		strcat(path, "bayonne.");
 		strcat(path, cp);
-		if(*cp != '/')
-			strcat(path, ".nmi");
-
-		new DSO(path);
-		cp = strtok(NULL, " \t\n;,");
+		pids[pidcount] = fork();
+		if(pids[pidcount] == 0)
+		{
+			argv[0] = path;
+			argv[1] = (char *)keypaths.getRunfiles();
+			argv[2] = NULL; 
+			execv(path, argv);
+			slog(Slog::levelError) << "libexec: " << path << ": unable to run" << endl;
+			exit(-1);
+		}
+		else if(pids[pidcount] > 0)
+			++pidcount;
+		else
+			slog(Slog::levelError) << "libexec: " << path << ": unable to fork" << endl;
 	}
 }
 
@@ -472,41 +667,22 @@ void Plugins::loadAuditing(void)
 	}
 }
 
-void Plugins::loadMaps(void)
-{
-	char path[256];
-	char list[512];
-	char *cp = (char *)getLast("maps");
-
-	if(!cp)
-		return;
-
-	strcpy(list, cp);
-	cp = strtok(list, " \t\n;,");
-	while(cp)
-	{
-		if(!*cp)
-			break;
-
-		strcpy(path, "maps/");
-		strcat(path, cp);
-		strcat(path, ".map");
-		if(canAccess(path))
-			new Map(path, cp);
-		cp = strtok(NULL, " \t\n;,");
-	}
-}
-
-void Plugins::loadFunctions(void)
+void Plugins::loadPreload(void)
 {
 	char path[256];
 	char list[512];
 	char *cp;
 
-	cp = (char *)getLast("functions");
+	cp = (char *)getLast("preload");
 	if(!cp)
 		return;
 
+	if(!stricmp(cp, "none"))
+		return;
+
+	if(!stricmp(cp, "no"))
+		return;
+
 	strcpy(list, cp);
 	cp = strtok(list, " \t\n;,");
 	while(cp)
@@ -514,17 +690,7 @@ void Plugins::loadFunctions(void)
 		if(!*cp)
 			break;
 
-		if(*cp == '/')
-			path[0] = 0;
-		else
-		{
-			strcpy(path, keypaths.getLibpath());
-			strcat(path, LIB_VERSION);
-		}
-		strcat(path, cp);
-		if(*cp != '/')
-			strcat(path, ".fun");
-		new DSO(path);
+		Script::use(cp);
 		cp = strtok(NULL, " \t\n;,");
 	}
 }
@@ -536,9 +702,24 @@ void Plugins::loadModules(void)
 	char *cp;
 
 	cp = (char *)getLast("modules");
+
 	if(!cp)
 		return;
 
+	if(!stricmp(cp, "no"))
+		return;
+
+	if(!stricmp(cp, "none"))
+		return;
+
+	if(!stricmp(cp, "*") || !stricmp(cp, "all"))
+	{
+		snprintf(path, sizeof(path), "%s/%s",
+			keypaths.getLibpath(), LIB_VERSION);
+		loader(path, ".mod");
+		return;
+	}
+
 	strcpy(list, cp);
 	cp = strtok(list, " \t\n;,");
 	while(cp)
@@ -549,10 +730,9 @@ void Plugins::loadModules(void)
 		if(*cp == '/')
 			path[0] = 0;
 		else
-		{
-			strcpy(path, keypaths.getLibpath());
-			strcat(path, LIB_VERSION);
-		}
+			snprintf(path, sizeof(path), "%s/%s",
+				keypaths.getLibpath(), LIB_VERSION);
+
 		strcat(path, cp);
 		if(*cp != '/')
 			strcat(path, ".mod");
@@ -561,17 +741,28 @@ void Plugins::loadModules(void)
 	}
 }
 
-
-void Plugins::loadFeeds(void)
+#ifdef	XML_SCRIPTS
+void Plugins::loadXML(void)
 {
 	char path[256];
 	char list[512];
 	char *cp;
 
-	cp = (char *)getLast("feeds");
+	cp = (char *)getLast("xml");
 	if(!cp)
 		return;
 
+	if(!stricmp(cp, "no") || !stricmp(cp, "none"))
+		return;
+
+	if(!stricmp(cp, "*") || !stricmp(cp, "all"))
+	{
+		snprintf(path, sizeof(path), "%s/%s",
+			keypaths.getLibpath(), LIB_VERSION);
+		loader(path, ".xml");
+		return;
+	}
+
 	strcpy(list, cp);
 	cp = strtok(list, " \t\n;,");
 	while(cp)
@@ -582,17 +773,17 @@ void Plugins::loadFeeds(void)
 		if(*cp == '/')
 			path[0] = 0;
 		else
-		{
-			strcpy(path, keypaths.getLibpath());
-			strcat(path, LIB_VERSION);
-		}
+			snprintf(path, sizeof(path), "%s/%s",
+				keypaths.getLibpath(), LIB_VERSION);
+
 		strcat(path, cp);
 		if(*cp != '/')
-			strcat(path, ".feed");
+			strcat(path, ".xml");
 		new DSO(path);
 		cp = strtok(NULL, " \t\n;,");
 	}
 }
+#endif
 
 void Plugins::loadTGI(void)
 {
@@ -621,23 +812,26 @@ void Plugins::loadTGI(void)
 		strcat(path, cp);
 		if(*cp != '/')
 			strcat(path, ".tgi");
-		new DSO(path);
+		dlopen(path, RTLD_LAZY | RTLD_GLOBAL);
+//		new DSO(path);
 		cp = strtok(NULL, " \t\n;,");
 	}
 }
 
 
-void Plugins::loadTranslators(void)
+void Plugins::loadTranslators(const char *lcp)
 {
 	char path[256];
 	char list[512];
 	char *cp;
 
-	cp = (char *)getLast("languages");
-	if(!cp)
+	if(!lcp)
+		lcp = getLast("languages");
+
+	if(!lcp)
 		return;
 
-	strcpy(list, cp);
+	strcpy(list, lcp);
 	cp = strtok(list, " \t\n;,");
 	while(cp)
 	{
@@ -690,8 +884,56 @@ void Plugins::loadDebug(void)
 		d = path;
 	}
 	new DSO(d);
+	if(!debug)
+	{
+		slog(Slog::levelNotice) << "no debug handler installed" << endl;
+		new Debug();
+	}
 }
-	
+
+void Plugins::loadMonitor(void)
+{
+	char path[256];
+	char *d = (char *)getLast("monitors");
+
+	if(!d)
+	{
+		new Monitor();
+		return;
+	}
+
+	if(*d != '/')
+	{
+		strcpy(path, keypaths.getLibpath());
+		strcat(path, LIB_VERSION);
+		strcat(path, d);
+		strcat(path, ".mon");
+		d = path;
+	}
+	new DSO(d);
+	if(!monitor)
+	{
+		slog(Slog::levelNotice) << "no monitor handler installed" << endl;
+		new Monitor();
+	}
+}
+
+void Plugins::loadSwitch(void)
+{
+	char path[256];
+	const char *sim = getLast("switch");
+
+	if(!sim)
+		return;
+
+	if(*sim)
+	{
+		snprintf(path, sizeof(path), "%s/%s/%s.sim",
+			keypaths.getLibpath(), LIB_VERSION, sim);
+		new DSO(path);
+	}
+}
+
 DSO *Plugins::loadDriver(void)
 {
 	char path[256];
@@ -740,19 +982,25 @@ TrunkGroup *TrunkGroup::first = NULL;
 TrunkGroup::TrunkGroup(char *name) :
 Keydata("/bayonne/trunks"), CallStat()
 {
-	KEYDEF keys[] = {
+	Keydata::Define keys[] = {
 		{"answer", "1"},
+		{"accept", "1"},
 		{"hangup", "100"},
+		{"siezetime", "12"},
 		{"ringtime", "7"},
 		{"flash", "200"},
-		{"dialtone", "800"},
+		{"dialtone", "2400"},
 		{"dialspeed", "160"},
 		{"volume", "80"},
 		{"callerid", "1500"},
 		{"pickup", "500"},
 		{"requests", "hangup"},
 		{"select", "last"},
+		{"threashold", "0"},
 		{"ready", "1000"},
+		{"idletime", "600"},
+		{"analysis", "16"},
+		{"dialmode", "dtmf"},
 		{NULL, NULL}};
 
 	int count = driver->getTrunkCount();
@@ -761,16 +1009,16 @@ Keydata("/bayonne/trunks"), CallStat()
 	char namebuf[65];
 	
 	schedule[0] = 0;
-	Load("~bayonne/trunks");
+	load("~bayonne/trunks");
 
 	if(name)
 	{
 		strcpy(namebuf, "/bayonne/");
 		strcat(namebuf, name);
 		strcat(namebuf, "-trunks");
-		Load(namebuf);
+		load(namebuf);
 		*namebuf = '~';
-		Load(namebuf);
+		load(namebuf);
 		cp = (char *)getLast("trunks");
 		if(cp)
 			cp = strtok(cp, " ,;\t\n");
@@ -781,6 +1029,28 @@ Keydata("/bayonne/trunks"), CallStat()
 				driver->groups[i] = this;
 			cp = strtok(NULL, " ,;\t\n");
 		}
+
+		cp = (char *)getLast("spans");
+		if(cp)
+			cp = strtok(cp, " ,;\t\n");
+		while(cp)
+		{
+			i = atoi(cp);
+			if(i > 0 && i < MAX_SPANS)
+				driver->spans[i] = this;
+			cp = strtok(NULL, " ,;\t\n");
+		}
+
+                cp = (char *)getLast("cards");
+                if(cp)
+                        cp = strtok(cp, " ,;\t\n");
+                while(cp)
+                {
+                        i = atoi(cp);
+                        if(i > 0 && i < MAX_CARDS)
+                                driver->cards[i] = this;
+                        cp = strtok(NULL, " ,;\t\n");
+                }
 	}
 	else
 	{
@@ -792,16 +1062,61 @@ Keydata("/bayonne/trunks"), CallStat()
 
 	setValue("name", name);
 
-	Load(keys);
+	load(keys);
 	next = NULL;
 	reqfirst = reqlast = NULL;
+	polFirst = NULL;
+	members = 0;
 
-	if(first)
-		first->next = this;
-	else
+	if(!first)
 		first = this;
+	else
+	{
+		next = first;
+		while(next->next)
+			next = next->next;
+		next->next = this;
+	}
+	next = NULL;
 }	
 
+bool TrunkGroup::getAccept(void)
+{
+	const char *cp = getLast("accept");
+	if(!cp)
+		return false;
+
+	switch(*cp)
+	{
+	case '0':
+	case 'f':
+	case 'F':
+	case 'n':
+	case 'N':
+		return false;
+	}
+	return true;
+}
+
+bool TrunkGroup::getDetect(void)
+{
+	const char *cp = getLast("detect");
+	if(!cp)
+		return false;
+
+	switch(*cp)
+	{
+	case '0':
+	case 'f':
+	case 'F':
+	case 'n':
+	case 'N':
+		return false;
+	}
+
+	return true;
+}
+
 seltype_t TrunkGroup::getSelect(void)
 {
 	const char *cp = getLast("select");
@@ -815,40 +1130,93 @@ seltype_t TrunkGroup::getSelect(void)
 Request *TrunkGroup::getRequest(void)
 {
 	Request *req = NULL;
+	Policy *pol;
+	Module *reply = Module::reqFirst;
+	Trunk *trunk;
+	TrunkEvent event;
 
-	EnterMutex();
+	enterMutex();
+	pol = polFirst;
+	while(pol)
+	{
+		req = pol->hiPriority();
+		if(req)
+		{
+			leaveMutex();
+			return req;
+		}
+		pol = pol->next;
+	}
 	while(reqfirst)
 	{
+		reply = Module::reqFirst;
 		if(reqfirst->isExpired())
 		{
-			slog(SLOG_WARNING) << "request: " << getName() << "(" << reqfirst->id << ") expired...";
+			slog(Slog::levelWarning) << "request: " << getName() << "(" << reqfirst->id << ") expired...";
+			trunk = driver->getTrunkId(reqfirst->parent);
+			if(trunk)
+			{
+				event.id = TRUNK_CHILD_FAIL;
+				trunk->postEvent(&event);
+			}
+			while(reply)
+			{
+				reply->expires(reqfirst);
+				reply = reply->reqNext;
+			}
 			delete reqfirst;
 			continue;
 		}
 	}
 	if(reqfirst)
 	{
+		reply = Module::reqFirst;
+		while(reply)
+		{
+			reply->running(reqfirst);
+			reply = reply->reqNext;
+		}
 		req = reqfirst;
-		req->Detach();
+		req->detach();
 	}
-	LeaveMutex();
+	pol = polFirst;
+	while(!req && pol)
+	{
+		req = pol->loPriority();
+		pol = pol->next;
+	}
+	leaveMutex();
 	return req;
 }
 
+const char *TrunkGroup::getRedirect(const char *redirect, char *buf)
+{
+	const char *cp = getLast(redirect);
+	if(cp)
+	{
+		strcpy(buf, cp);
+		return buf;
+	}
+	return getSchedule(buf);
+}
+
 const char *TrunkGroup::getSchedule(char *buf)
 {
 	const char *scr = getLast("script");	// check for .conf override
 	if(scr)
-		return scr;
+	{
+		strcpy(buf, scr);
+		return buf;
+	}
 
-	scheduler.ReadLock();
+	scheduler.readLock();
 	if(schedule[0])
 		strcpy(buf, schedule);
 	else if(this == first)
 		strcpy(buf, keyserver.getDefault());
 	else
 		strcpy(buf, getName());
-	scheduler.Unlock();
+	scheduler.unlock();
 	return buf;
 }
 
@@ -867,10 +1235,10 @@ void TrunkGroup::setSchedule(const char *str)
 	if(!str)
 		str = "";
 
-	scheduler.WriteLock();
+	scheduler.writeLock();
 	strncpy(schedule, str, sizeof(schedule) - 1);
 	schedule[sizeof(schedule) - 1] = 0;
-	scheduler.Unlock();
+	scheduler.unlock();
 }
 
 phTone *phTone::first = NULL;
@@ -893,6 +1261,22 @@ int phTone::ulaw[256] = {
         7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
         7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};
 
+unsigned char phTone::linear2ulaw(int sample)
+{
+	int sign, exponent, mantissa, retval;
+
+	sign = (sample >> 8) & 0x80;
+	if(sign != 0) sample = -sample;
+	sample += 0x84;
+	exponent = ulaw[(sample >> 7) & 0xff];
+	mantissa = (sample >> (exponent + 3)) & 0x0f;
+	retval = ~(sign | (exponent << 4) | mantissa);
+	if(!retval)
+		retval = 0x02;
+
+	return retval;
+}
+
 phTone::phTone(const char *n, timeout_t dur, unsigned f) 
 {
 	int i, sign, exponent, mantissa;
@@ -913,16 +1297,9 @@ phTone::phTone(const char *n, timeout_t dur, unsigned f)
 
 	for(i = 0; i < dur * 8; ++i)
 	{
-		sample = (int)(sin(pos) * 30000);
+		sample = (int)(sin(pos) * 20000.0);
 		pos += freq;
-		sign = (sample >> 8) & 0x80;
-		if(sign != 0) sample = -sample;
-		sample += 0x84;
-		exponent = ulaw[(sample >> 7) & 0xff];
-		mantissa = (sample >> (exponent + 3)) & 0x0f;
-		samples[i] = -(sign | (exponent << 4) | mantissa);
-		if(!samples[i])
-			samples[i] = 0x02;
+		samples[i] = linear2ulaw(sample);
 	}
 }
 
@@ -947,17 +1324,10 @@ phTone::phTone(const char *n, timeout_t dur, unsigned f1, unsigned f2)
 
 	for(i = 0; i < dur * 8; ++i)
 	{
-		sample = (int)((sin(pos1) + sin(pos2)) * 15000);
+		sample = (int)((sin(pos1) + sin(pos2)) * 10000.0);
 		pos1 += fa1;
 		pos2 += fa2;
-		sign = (sample >> 8) & 0x80;
-		if(sign != 0) sample = -sample;
-		sample += 0x84;
-		exponent = ulaw[(sample >> 7) & 0xff];
-		mantissa = (sample >> (exponent + 3)) & 0x0f;
-		samples[i] = -(sign | (exponent << 4) | mantissa);
-		if(!samples[i])
-			samples[i] = 0x02;
+		samples[i] = linear2ulaw(sample);
 	}
 }
 
@@ -969,7 +1339,7 @@ phTone::~phTone()
 }
 
 
-void phTone::Clear(void)
+void phTone::clear(void)
 {
 	if(samples)
 		delete samples;
@@ -993,6 +1363,9 @@ TrunkGroup *getGroup(const char *name)
 {
 	TrunkGroup *group = TrunkGroup::first;
 
+	if(!name)
+		return group;
+
 	while(group)
 	{
 		if(!stricmp(name, group->getName()))
@@ -1002,18 +1375,31 @@ TrunkGroup *getGroup(const char *name)
 	return NULL;
 }		
 
+Keydata application("/bayonne/application");
+#ifdef	USER_HOSTING
+Keydata keyusers;
+#endif
 KeyServer keyserver;
 KeyPaths keypaths;
 KeyLocal keylocal;
-KeyMailbox keymailbox;
+KeyHandlers keyhandlers;
+#ifdef	NODE_SERVICES
 KeyNetwork keynetwork;
+#endif
 KeyTones keytones;
 KeyThreads keythreads;
 KeyMemory keymemory;
-KeyFeed keyfeed;
+#ifdef	XML_SCRIPTS
+KeyProxy keyproxy;
+#endif
 Plugins plugins;
 Scheduler scheduler;
+#ifdef	NODE_SERVICES
 Network network;
+#endif
 
 ScriptSymbol Trunk::globals(keymemory.getSymbolSize(), keymemory.getPageSize());
 
+#ifdef	CCXX_NAMESPACES
+};
+#endif
