@@ -1,4 +1,4 @@
-// Copyright (C) 2000 Open Source Telecom Corporation.
+// Copyright (C) 2000-2001 Open Source Telecom Corporation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,9 +15,13 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 #include "server.h"
-#include <ctype.h>
+#include <cc++/strchar.h>
+
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
 
-FifoHandler *FifoHandler::first = NULL;
 Fifo fifo;
 
 static Trunk *getTrunk(char *str)
@@ -43,6 +47,7 @@ bool Fifo::waitPid(char **argv)
 
 bool Fifo::setSymbol(char **argv)
 {
+	Script::Symbol *sym;
 	Trunk *trunk = getTrunk(argv[1]);
 	if(!trunk)
 		return false;
@@ -50,31 +55,72 @@ bool Fifo::setSymbol(char **argv)
 	if(!argv[3] || !argv[2])
 		return false;
 
-	trunk->setSymbol(argv[2], trunk->getSymbolSize());
-	trunk->setSymbol(argv[2], argv[3]);
+	trunk->enterMutex();
+	sym = trunk->getLocal(argv[2], trunk->getSymbolSize());
+	if(sym)
+	{
+		if(!sym->flags.readonly)
+		{
+			snprintf(sym->data, sym->flags.size + 1, "%s",  argv[3]);
+			if(sym->flags.commit)
+				trunk->commit(sym);
+		}
+		
+	}
+	trunk->leaveMutex();
+//	trunk->setSymbol(argv[2], trunk->getSymbolSize());
+//	trunk->setSymbol(argv[2], argv[3]);
 	return true;
 }
 
+bool Fifo::reload(char **argv)
+{
+	Module *mod = Module::cmdFirst;
+	int argc = 1;
+	bool rts = true;
+
+	if(!argv[1])
+	{
+		while(mod)
+		{
+			mod->reload();
+			mod = mod->cmdNext;
+		}
+		return true;
+	}
+
+	while(argv[argc])
+	{
+		mod = getModule(MODULE_FIFO, argv[argc++]);
+		if(mod)
+			mod->reload();
+		else
+			rts = false;
+	}
+	return rts;
+}
+
 bool Fifo::submit(char **argv)
 {
 	Trunk *trunk = getTrunk(argv[1]);
-	char buffer[512];
-	strstream str(buffer, 512);
+	char buffer[PIPE_BUF / 2];
 	TrunkEvent event;
 	int argc = 3;
 	unsigned seq;
+	int len = 0;
 
 	if(!trunk || !argv[2])
 		return false;
 
 	seq = atoi(argv[2]);
-	while(argv[argc])
+	while(argv[argc] && len < sizeof(buffer))
 	{
 		if(argc > 3)
-			str << '&';
-		str << argv[argc++];
+			buffer[len++] = '&';
+		strncpy(buffer + len, argv[argc++], sizeof(buffer) - len);
+		buffer[sizeof(buffer) - 1] = 0;
+		len = strlen(buffer);
 	}
-	str << ends;
 	event.id = TRUNK_SERVICE_LOOKUP;
 	event.parm.lookup.seq = seq;
 	event.parm.lookup.result = true;
@@ -82,6 +128,35 @@ bool Fifo::submit(char **argv)
 	return trunk->postEvent(&event);
 }
 
+bool Fifo::setLimit(char **argv)
+{
+	Mixer *mixer = driver->getMixer(atoi(argv[1]));
+	Conference *conf;
+	int grp = 0;
+	int argc = 2;
+	bool rts = true;
+
+	if(!mixer || !argv[2])
+		return false;
+
+	while(argv[argc] && rts)
+	{
+		conf = mixer->getConference(grp++);
+		if(!conf)
+			return false;
+		rts = conf->setConference(atoi(argv[argc++]));
+	}
+	return rts;
+}
+
+bool Fifo::setMixer(char **argv)
+{
+	Mixer *mixer = driver->getMixer(atoi(argv[1]));
+	if(!mixer || !argv[2] || !argv[3])
+		return false;
+
+	return mixer->setMixer(atoi(argv[2]), atoi(argv[3]));
+}
 
 bool Fifo::ringScript(char **argv)
 {
@@ -97,11 +172,22 @@ bool Fifo::ringScript(char **argv)
 	return trunk->postEvent(&event);
 }
 
+bool Fifo::redirectScript(char **argv)
+{
+	Trunk *trunk = getTrunk(argv[1]);
+	TrunkEvent event;
+
+	if(!trunk || !argv[2])
+		return false;
+
+	event.id = TRUNK_RING_REDIRECT;
+	event.parm.argv = &argv[2];
+
+	return trunk->postEvent(&event);
+}
+
 bool Fifo::reqScript(char **argv)
 {
-	int argc = 4;
-	char buffer[512];
-	strstream str(buffer, sizeof(buffer));
 	TrunkGroup *grp;
 	char *exp = argv[1];
 	
@@ -112,28 +198,95 @@ bool Fifo::reqScript(char **argv)
 	if(!grp || !argv[3])
 		return false;
 
-	buffer[0] = 0;
-	str << argv[3];
-	while(argv[argc])
-		str << " " << argv[argc++];
-
-	str << ends;
-	new Request(grp, buffer, atoi(exp));
+	request(grp, &argv[3], atoi(exp));
 	return true;
 }
 
 bool Fifo::startScript(char **argv)
 {
-	Trunk *trunk = getTrunk(argv[1]);	
+	Trunk *trunk = getTrunk(argv[1]);
+	TrunkGroup *group = getGroup(argv[1]);	
 	TrunkEvent event;
+	int port = driver->getTrunkCount();
 	
+	if((!trunk && !group) || !argv[2])
+		return false;
+
+	if(trunk && !group)
+	{
+		event.id = TRUNK_START_SCRIPT;
+		event.parm.argv = &argv[2];
+		return trunk->postEvent(&event);
+	}
+	if(!group)
+		return false;
+
+	while(port--)
+	{
+		if(driver->getTrunkGroup(port) != group)
+			continue;
+
+		trunk = driver->getTrunkPort(port);
+		if(!trunk)
+			continue;
+
+		event.id = TRUNK_START_SCRIPT;
+		event.parm.argv = &argv[2];
+		if(trunk->postEvent(&event))
+			return true;
+	}
+	return false;
+}
+
+bool Fifo::postKey(char **argv)
+{
+	Trunk *trunk = getTrunk(argv[1]);
+	char *digits = argv[2];
+	TrunkEvent event;
+	bool rtn = true;
+
 	if(!trunk || !argv[2])
 		return false;
 
-	event.id = TRUNK_START_SCRIPT;
-	event.parm.argv = &argv[2];
-
-	return trunk->postEvent(&event);
+	while(digits && rtn)
+	{
+		event.id = TRUNK_DTMF_KEYUP;
+		event.parm.dtmf.duration = 40;
+		event.parm.dtmf.e1 = event.parm.dtmf.e2 = 0;
+		switch(*digits)
+		{
+		case '0':
+		case '1':
+		case '2':
+		case '3':
+		case '4':
+		case '5':
+		case '6':
+		case '7':
+		case '8':
+		case '9':
+			event.parm.dtmf.digit = *digits - '0';
+			rtn = trunk->postEvent(&event);
+			break;
+		case '*':
+			event.parm.dtmf.digit = 10;
+			rtn = trunk->postEvent(&event);
+			break;
+		case '#':
+			event.parm.dtmf.digit = 11;
+			rtn = trunk->postEvent(&event);
+			break; 
+		default:
+			rtn = false;
+		}
+		if(*(++digits))
+#ifdef	COMMON_THREAD_SLEEP
+			Thread::sleep(60);
+#else
+			ccxx_sleep(60);
+#endif
+	}
+	return rtn;
 }
 
 bool Fifo::exitPid(char **argv)
@@ -214,6 +367,59 @@ bool Fifo::busyLine(char **argv)
 	return true;
 }
 
+bool Fifo::setSpan(char **argv)
+{
+	TrunkEvent event;
+	Trunk *trunk;
+	unsigned span;
+	unsigned port;
+
+	const char *cspan = argv[1];
+	const char *mode = argv[2];
+
+	if(!cspan || !mode)
+		return false;
+
+	span = atoi(cspan);
+	if(!stricmp(mode, "busy"))
+		event.id = TRUNK_MAKE_BUSY;
+	else if(!stricmp(mode, "idle") || !stricmp(mode, "up"))
+		event.id = TRUNK_MAKE_IDLE;
+	else if(!stricmp(mode, "stop") || !stricmp(mode, "down"))
+		event.id = TRUNK_MAKE_STANDBY;
+	else
+		return false;
+
+	return driver->spanEvent(span, &event);
+}
+
+
+bool Fifo::setCard(char **argv)
+{
+        TrunkEvent event;
+        Trunk *trunk;
+        unsigned card;
+        unsigned port;
+
+        const char *ccard = argv[1];
+        const char *mode = argv[2];
+
+        if(!ccard || !mode)
+                return false;
+
+        card = atoi(ccard);
+        if(!stricmp(mode, "busy"))
+                event.id = TRUNK_MAKE_BUSY;
+        else if(!stricmp(mode, "idle") || !stricmp(mode, "up"))
+                event.id = TRUNK_MAKE_IDLE;
+        else if(!stricmp(mode, "stop") || !stricmp(mode, "down"))
+                event.id = TRUNK_MAKE_STANDBY;
+        else
+                return false;
+
+        return driver->cardEvent(card, &event);
+}
+
 bool Fifo::idleLine(char **argv)
 {
 	TrunkGroup *group = getGroup(argv[1]);
@@ -245,45 +451,43 @@ bool Fifo::idleLine(char **argv)
 	return true;
 }
 
-bool Fifo::mapFiles(char **argv)
+bool Fifo::setSchedule(char **argv)
 {
-	int count = 0;
-	char pathname[128];
-
-	while(*(++argv))
+	if(argv[1])
 	{
-		strcpy(pathname, "maps/");
-		strcat(pathname, *argv);
-		strcat(pathname, ".map");
-		if(!canAccess(pathname))
-			continue;
-		new Map(pathname, *argv);
-		++count;
+		strcpy(schedule, argv[1]);
+		if(!stricmp(schedule, "*"))
+			scheduler.altSchedule(NULL);
+		else
+			scheduler.altSchedule(schedule);
 	}
-	if(count)
-		return true;
 	else
-		return false;
+		scheduler.altSchedule(NULL);
+	return true;
 }
 		
-bool Fifo::Command(const char *cmd)
+bool Fifo::command(const char *cmd, ostream *fd)
 {
-	FifoHandler *next;
-	char buffer[512];
+	Module *next;
+	char buffer[PIPE_BUF / 2];
 	bool rts = false;
-	char *argv[65];
+	char *args[65];
 	int argc = 0;
+	char **argv = args;
 	char *arg;
 	char *sp;
 	char *ptr;
 	const char *token = keyserver.getToken();
 	int tlen = strlen(token);
 
+	if(!fd)
+		fd = &slog;
+
 	strncpy(buffer, cmd, sizeof(buffer) - 1);
 	buffer[sizeof(buffer) - 1] = 0;
 
-	EnterMutex();
-	slog(SLOG_DEBUG) << "fifo: cmd=" << cmd << endl;
+	enterMutex();
+	slog(Slog::levelDebug) << "fifo: cmd=" << cmd << endl;
 	if(strstr(buffer, token))
 	{
 		ptr = buffer;
@@ -317,26 +521,37 @@ bool Fifo::Command(const char *cmd)
 
 	if(!argv[0])
 	{
-		LeaveMutex();
+		leaveMutex();
 		return false;
 	}
 
 	if(!*argv[0])
 	{
-		LeaveMutex();
+		leaveMutex();
 		return false;
 	}
 
-	next = FifoHandler::first;
+	next = Module::cmdFirst;
 
 	while(next && !rts)
 	{
-		rts = next->Command(argv);
-		next = next->next;
+		rts = next->command(argv, fd);
+		next = next->cmdNext;
 	}
 
 	if(!stricmp(argv[0], "down"))
 	{
+		if(argv[1])
+			snprintf(service, sizeof(service), "down::%s", argv[1]);
+		else
+			raise(SIGINT);
+		rts = true;
+	}
+	else if(!stricmp(argv[0], "up"))
+		service[0] = 0;
+	else if(!stricmp(argv[0], "restart"))
+	{
+		restart_server = true;
 		raise(SIGINT);
 		rts = true;
 	}
@@ -354,28 +569,40 @@ bool Fifo::Command(const char *cmd)
 	else if(!stricmp(argv[0], "submit"))
 		rts = submit(argv);
 	else if(!stricmp(argv[0], "debug"))
-		rts = debug->DebugFifo(argv);
+		rts = debug->debugFifo(argv);
 	else if(!stricmp(argv[0], "ring"))
 		rts = ringScript(argv);
+	else if(!stricmp(argv[0], "redirect"))
+		rts = redirectScript(argv);
 	else if(!stricmp(argv[0], "busy"))
 		rts = busyLine(argv);
 	else if(!stricmp(argv[0], "idle"))
 		rts = idleLine(argv);
+	else if(!stricmp(argv[0], "span"))
+		rts = setSpan(argv);
+	else if(!stricmp(argv[0], "card"))
+		rts = setCard(argv);
 	else if(!stricmp(argv[0], "start"))
 		rts = startScript(argv);	
 	else if(!stricmp(argv[0], "request"))
 		rts = reqScript(argv);
 	else if(!stricmp(argv[0], "disconnect") || !stricmp(argv[0], "hangup"))
 		rts = hangupLine(argv);
-	else if(!stricmp(argv[0], "map"))
-		rts = mapFiles(argv);
-	LeaveMutex();
+	else if(!stricmp(argv[0], "post") || !stricmp(argv[0], "key"))
+		rts = postKey(argv);
+	else if(!stricmp(argv[0], "schedule"))
+		rts = setSchedule(argv);
+	else if(!stricmp(argv[0], "mixer"))
+		rts = setMixer(argv);
+	else if(!stricmp(argv[0], "limit"))
+		rts = setLimit(argv);
+	else if(!stricmp(argv[0], "reload"))
+		rts = reload(argv);
+	leaveMutex();
 	return rts;
 }
 
-FifoHandler::FifoHandler()
-{
-	next = first;
-	first = this;
-}
 
+#ifdef	CCXX_NAMESPACES
+};
+#endif
