@@ -1,4 +1,4 @@
-// Copyright (C) 2000 Open Source Telecom Corporation.
+// Copyright (C) 2000-2001 Open Source Telecom Corporation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,6 +16,11 @@
 
 #include "server.h"
 
+#ifdef	CCXX_NAMESPACES
+namespace ost {
+using namespace std;
+#endif
+
 CallStat::CallStat() :
 Mutex()
 {
@@ -24,55 +29,58 @@ Mutex()
 	active.incoming = active.outgoing = max.incoming = max.outgoing = lastmax.incoming = lastmax.outgoing = 0;
 }
 
-void CallStat::Update(void)
+void CallStat::update(void)
 {
-	EnterMutex();
+	enterMutex();
 	memcpy(&lasttotal, &total, sizeof(total));
 	memcpy(&lastmax, &max, sizeof(max));
 	memcpy(&max, &active, sizeof(active));
 	memset(&total, 0, sizeof(total));
-	LeaveMutex();
+	leaveMutex();
 }
 
 void CallStat::incIncoming(void)
 {
-	EnterMutex();
+	enterMutex();
 	++total.incoming;
 	if(++active.incoming > max.incoming)
 		++max.incoming;
-	LeaveMutex();
+	leaveMutex();
 }
 
 void CallStat::decIncoming(void)
 {
-	EnterMutex();
+	enterMutex();
 	--active.incoming;
-	LeaveMutex();
+	leaveMutex();
 }
 
 void CallStat::incOutgoing(void)
 {
-	EnterMutex();
+	enterMutex();
 	++total.outgoing;
 	if(++active.outgoing > max.outgoing)
 		++max.outgoing;
-	LeaveMutex();
+	leaveMutex();
 }
 
 void CallStat::decOutgoing(void)
 {
-	EnterMutex();
+	enterMutex();
 	--active.outgoing;
-	LeaveMutex();
+	leaveMutex();
 }
 
 long CallStat::getStat(statitem_t item)
 {
 	long value = 0;
 
-	EnterMutex();
+	enterMutex();
 	switch(item)
 	{
+	case STAT_ACTIVE_CALLS:
+		value = lasttotal.outgoing + lasttotal.incoming;
+		break;
 	case STAT_MAX_INCOMING:
 		value = lastmax.incoming;
 		break;
@@ -86,7 +94,10 @@ long CallStat::getStat(statitem_t item)
 		value = lasttotal.outgoing;
 		break;
 	}
-	LeaveMutex();
+	leaveMutex();
 	return value;
 }
 
+#ifdef	CCXX_NAMESPACES
+};
+#endif
